@@ -104,7 +104,8 @@ contain =function (s1, s2) {
         matched=FALSE
         for (i in 1:(nchar(s)-k+1) ) {
             if (substr(s, i, i+k-1)==s2) {
-                matched=TRUE
+                matched=i
+                break
             }
         }
         matched
